#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


## Wing-body-tail configuration file for flow trim script
# Adrien Crovato
#
# CAUTION
# Gmsh might not generate the wing/tail surfaces properly. Results might not be accurate

def getParam():
    p = {}
    # Specific
    scl = 2 # scaling factor for lifting surface mesh size
    wrlems = scl*0.0210 # wing root leading edge mesh size
    wrtems = scl*0.0210 # wing root trailing edge mesh size
    wtlems = scl*0.0090 # wing tip leading edge mesh size
    wttems = scl*0.0090 # wing tip trailing edge mesh size
    wwlems = scl*0.0042 # wing winglet leading mesh size
    wwtems = scl*0.0060 # wing winglet trailing mesh size
    trlems = scl*0.0089 # tail root leading mesh size
    trtems = scl*0.0089 # tail root trailing mesh size
    ttlems = scl*0.0058 # tail tip leading mesh size
    tttems = scl*0.0058 # tail tip trailing mesh size
    fums = 0.15 # fuselage mesh size
    fucms = 0.01 # fuselage caps mesh size
    fams = 5. # farfield mesh size
    # Input/Output
    p['File'] = '../models/wbht.geo' # Input file containing the model
    p['Pars'] = {'msLeW0' : wrlems, 'msTeW0' : wrtems,
                 'msLeW1' : wtlems, 'msTeW1' : wttems,
                 'msLeW2' : wwlems, 'msTeW2' : wwtems,
                 'msLeT0' : trlems, 'msTeT0' : trtems,
                 'msLeT1' : ttlems, 'msTeT1' : tttems,
                 'msFus' : fums, 'msFusCaps' : fucms, 'msFar' : fams} # Parameters for input file model
    p['Dim'] = 3 # Problem dimension
    p['Format'] = 'vtk' # Save format (vtk or gmsh)
    p['Slice'] = [0.83, 3.0, 4.0, 6.2, 6.9] # array of (y) coordinates to perform slice along the span (empty if none)
    p['TagId'] = 10 # tag number of physical group to be sliced
    # Markers
    p['Fluid'] = 'field' # Name of physical group containing the fluid
    p['Farfield'] = ['upstream', 'farfield', 'downstream'] # LIST of names of physical groups containing the farfield boundaries (upstream/downstream should be first/last element)
    p['Symmetry'] = 'symmetry' # Name of physical group containing the symmetry boundaries
    p['Fuselage'] = 'fuselage' # Name of physical group containing the fuselage boundaries
    p['Wings'] = ['wing', 'tail'] # LIST of names of physical groups containing the lifting surface boundary
    p['Wakes'] = ['wakeW', 'wakeT'] # LIST of names of physical group containing the wake
    p['WakeTips'] = ['wakeTipW', 'wakeTipT'] # LIST of names of physical group containing the edge of the wake
    p['TeTips'] = ['teTipW', 'teTipT'] # LIST of names of physical group containing the edge of the wake and the trailing edge
    # Freestream
    p['M_inf'] = 0.75 # Freestream Mach number
    p['CL'] = 0.4 # Target lift coefficient
    p['AoA'] = 0. # Freestream angle of attack (guess)
    p['dCL'] = 7.5 # Slope of CL w.r.t AoA [1/rad] (guess)
    # Geometry
    p['S_ref'] = 8.9 # Reference surface length
    p['c_ref'] = 1.7 # Reference chord length
    p['x_ref'] = 4.74 # Reference point for moment computation (x)
    p['y_ref'] = 0.66 # Reference point for moment computation (y)
    p['z_ref'] = -0.5 # Reference point for moment computation (z)
    # Numerical
    p['LSolver'] = 'GMRES' # Linear solver (Pardiso, GMRES, MUMPS or SparseLU)
    p['G_fill'] = 2 # Fill-in factor for GMRES preconditioner
    p['G_tol'] = 1e-5 # Tolerance for GMRES
    p['G_restart'] = 50 # Restart for GMRES
    p['NSolver'] = 'Newton' # Noninear solver type (Picard or Newton)
    p['Rel_tol'] = 1e-6 # Relative tolerance on solver residual
    p['Abs_tol'] = 1e-8 # Absolute tolerance on solver residual
    p['Max_it'] = 15 # Solver maximum number of iterations
    p['LS_tol'] = 1e-6 # Tolerance on linesearch residual
    p['Max_it_LS'] = 10 # Linesearch maximum number of iterations
    p['AV_thrsh'] = 1e-2 # Residual threshold below which the artificial viscosity is decreased
    return p

def main():
    # Script call
    import dart.scripts.trim as t
    trim = t.Trim(getParam())
    trim.run()
    trim.disp()

if __name__ == "__main__":
    main()
